"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toKeyframeString = toKeyframeString;
exports.default = makeKeyframe;

var _styledComponents = require("styled-components");

var _tools = require("@bootstrap-styled/utils/lib/tools");

/**
 * This will store a reference to all the keyframe created with makeKeyFrame
 * @type {*[]}
 */
var keyframeRefList = [];
/**
 * @public
 * @name toKeyframeString
 * @description This will convert an object into a valid keyframe string for styled
 * @param keyFrameObject {Object} - Object of keyFrames
 * @example
 * const kf = {
 *   from: {
 *     width: '0%',
 *   },
 *   '20%, 40%': {
 *     width: '75%',
 *   },
 *   '60%': {
 *     width: '25%',
 *   },
 *   to: {
 *     width: '100%',
 *   },
 * };
 * toKeyframeString(kf);
 * // return the key frame stringified
 * @returns {string} - The keyframe stringified
 */

function toKeyframeString(keyFrameObject) {
  var keyframeStr = '';
  Object.keys(keyFrameObject).forEach(function (step) {
    keyframeStr += "".concat(step, "{");
    Object.keys(keyFrameObject[step]).forEach(function (cssAttr) {
      keyframeStr += "".concat(cssAttr, ":").concat(keyFrameObject[step][cssAttr], ";");
    });
    keyframeStr += '}';
  });
  return keyframeStr;
}
/**
 * @public
 * @name makeKeyframe
 * @description make styled keyframe
 * @param make {function} - make animation function
 * @param options {object} - options
 * @param userKeyframes {object} - user keyframes
 * @param maxKeyframe {number} [maxKeyframe=50] - max key frame injected in the page before warning
 * @returns {string} the name of the keyframe
 */


function makeKeyframe(make, options) {
  var userKeyframes = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var maxKeyframe = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 50;
  var merge = make(options);
  Object.keys(userKeyframes).forEach(function (key) {
    merge[key] = Object.assign({}, merge[key], userKeyframes[key]);
  });
  var hashCode = (0, _tools.toHashCode)(JSON.stringify(merge));
  var filtered = keyframeRefList.filter(function (keyframeRef) {
    return keyframeRef.hashCode === hashCode;
  });

  if (filtered.length) {
    return filtered[0].name;
  }

  var keyframeStr = toKeyframeString(merge);
  var name = (0, _styledComponents.keyframes)(["", ""], keyframeStr);
  keyframeRefList.push({
    name: name,
    hashCode: hashCode
  });
  /* istanbul ignore if */

  if (keyframeRefList.length > maxKeyframe) {
    console.warn("You might have done a mistake because of current keyframes injection count. You currently have ".concat(keyframeRefList.length, " keyframes in your page.")); // eslint-disable-line no-console
  }

  return name;
}